#ifndef DEVICECONTROLLERINTERFACE_H
#define DEVICECONTROLLERINTERFACE_H

#include <QtPlugin>

#include <QSettings>
#include <QString>

#include "ATBAPPplugin.h"
#include "UnifiedDCVMCInterface.h"

namespace nsDeviceControllerInterface {
    using PLUGIN_STATE = ATBAPPplugin::PLUGIN_STATE;
    using RESULT_STATE = ATBAPPplugin::RESULT_STATE;
    using CASH_STATE = ATBAPPplugin::CASH_STATE;
    using TICKET_VARIANT = ATBAPPplugin::TICKET_VARIANT;
}

class DeviceControllerInterface : public UnifiedDCVMCInterface
{
    Q_OBJECT
    Q_INTERFACES(ATBAPPplugin)
    Q_INTERFACES(UnifiedDCVMCInterface)

public:

    virtual PLUGIN_STATE initPlugin(QObject *eventReceiver, QSettings const &settings) override {
        return initDCPlugin(eventReceiver, settings);
    }

    virtual ~DeviceControllerInterface() {}

    /**
     * @brief initDCPlugin
     * @param eventReceiver - QObject to receive ATBMachineEvents or HealthEvents
     * @param settings
     * @return
     */
    virtual PLUGIN_STATE initDCPlugin(QObject *eventReceiver, const QSettings & settings) = 0;

    // TASKS: Cash handling -------------------------------------------------------
    /**
     * enables coin input
     * amount = "0": pay-up
     * amount > "0": pay-down
     */
    virtual void requestStartCashInput(const QString & amount) = 0;

    /**
     * called e.g. on Button "NEXT" in pay-up (direct coin input)
     */
    virtual void requestStopCashInput() = 0;

    /**
     * called e.g. on Button "NEXT" in pay-up (direct coin input)
     */
    virtual void cashCollect() = 0;
    virtual void cashAbort() = 0;

    // TASKS: Account -------------------------------------------------------------

    virtual void requestAccount() = 0;


    // TASKS: printing ------------------------------------------------------------
    virtual void requestPrintTicket(const QHash<QString, QVariant> & printingData) = 0;
    virtual void requestPrintTicket(TICKET_VARIANT ticketVariant, const QHash<QString, QVariant> & printingData) = 0;
    virtual void requestPrintReceipt(const QHash<QString, QVariant> & printingData) = 0;
    virtual void requestPrintReceipt(const QString & printingString) = 0;
};

Q_DECLARE_INTERFACE(DeviceControllerInterface,
                    "eu.atb.ptu.plugin.DeviceControllerInterface/1.0")

#endif // DEVICECONTROLLERINTERFACE_H
